@@ -13,11 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
 import datetime
 import decimal
 
+from oslo_config import cfg
 from oslo_log import log as logging
 import pecan
 from pecan import rest
@@ -26,10 +25,15 @@ import wsmeext.pecan as wsme_pecan
 
 from cloudkitty.api.v1.datamodels import report as report_models
 from cloudkitty.common import policy
+from cloudkitty import tzutils
 from cloudkitty import utils as ck_utils
 
 LOG = logging.getLogger(__name__)
 
+CONF = cfg.CONF
+
+CONF.import_opt('scope_key', 'cloudkitty.collector', 'collect')
+
 
 class InvalidFilter(Exception):
     """Exception raised when a storage filter is invalid"""
@@ -94,18 +98,18 @@ class ReportController(rest.RestController):
         # FIXME(sheeprine): We should filter on user id.
         # Use keystone token information by default but make it overridable and
         # enforce it by policy engine
-        groupby = ['project_id']
-        group_filters = {'project_id': tenant_id} if tenant_id else None
-        total_resources = storage.total(
+        scope_key = CONF.collect.scope_key
+        groupby = [scope_key]
+        filters = {scope_key: tenant_id} if tenant_id else None
+        result = storage.total(
             groupby=groupby,
             begin=begin, end=end,
             metric_types=service,
-            group_filters=group_filters)
+            filters=filters)
 
-        # TODO(Aaron): `get_total` return a list of dict,
-        # Get value of rate from index[0]
-        total = sum(total['rate'] for total in total_resources)
-        return total if total else decimal.Decimal('0')
+        if result['total'] < 1:
+            return decimal.Decimal('0')
+        return sum(total['rate'] for total in result['results'])
 
     @wsme_pecan.wsexpose(report_models.SummaryCollectionModel,
                          datetime.datetime,
@@ -133,25 +137,26 @@ class ReportController(rest.RestController):
                          {"tenant_id": tenant_id})
         storage = pecan.request.storage_backend
 
+        scope_key = CONF.collect.scope_key
         storage_groupby = []
         if groupby is not None and 'tenant_id' in groupby:
-            storage_groupby.append('project_id')
+            storage_groupby.append(scope_key)
         if groupby is not None and 'res_type' in groupby:
             storage_groupby.append('type')
-        group_filters = {'project_id': tenant_id} if tenant_id else None
-        results = storage.total(
+        filters = {scope_key: tenant_id} if tenant_id else None
+        result = storage.total(
             groupby=storage_groupby,
             begin=begin, end=end,
             metric_types=service,
-            group_filters=group_filters)
+            filters=filters)
 
         summarymodels = []
-        for res in results:
+        for res in result['results']:
             kwargs = {
                 'res_type': res.get('type') or res.get('res_type'),
-                'tenant_id': res.get('project_id') or res.get('tenant_id'),
-                'begin': res['begin'],
-                'end': res['end'],
+                'tenant_id': res.get(scope_key) or res.get('tenant_id'),
+                'begin': tzutils.local_to_utc(res['begin'], naive=True),
+                'end': tzutils.local_to_utc(res['end'], naive=True),
                 'rate': res['rate'],
             }
             summarymodel = report_models.SummaryModel(**kwargs)
