@@ -13,11 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
 import datetime
-import decimal
 
+from oslo_config import cfg
 import pecan
 from pecan import rest
 from wsme import types as wtypes
@@ -26,7 +24,12 @@ import wsmeext.pecan as wsme_pecan
 from cloudkitty.api.v1.datamodels import storage as storage_models
 from cloudkitty.common import policy
 from cloudkitty import storage
-from cloudkitty import utils as ck_utils
+from cloudkitty import tzutils
+
+
+CONF = cfg.CONF
+
+CONF.import_opt('scope_key', 'cloudkitty.collector', 'collect')
 
 
 class DataFramesController(rest.RestController):
@@ -48,47 +51,48 @@ class DataFramesController(rest.RestController):
         :return: Collection of DataFrame objects.
         """
 
-        policy.authorize(pecan.request.context, 'storage:list_data_frames', {})
+        project_id = tenant_id or pecan.request.context.project_id
+        policy.authorize(pecan.request.context, 'storage:list_data_frames', {
+            'tenant_id': project_id,
+        })
 
+        scope_key = CONF.collect.scope_key
         backend = pecan.request.storage_backend
         dataframes = []
-        group_filters = {'project_id': tenant_id} if tenant_id else None
-
-        if begin:
-            begin = ck_utils.dt2ts(begin)
-        if end:
-            end = ck_utils.dt2ts(end)
+        if pecan.request.context.is_admin:
+            filters = {scope_key: tenant_id} if tenant_id else None
+        else:
+            filters = {scope_key: project_id}
         try:
             resp = backend.retrieve(
                 begin, end,
-                group_filters=group_filters,
+                filters=filters,
                 metric_types=resource_type,
                 paginate=False)
         except storage.NoTimeFrame:
             return storage_models.DataFrameCollection(dataframes=[])
         for frame in resp['dataframes']:
-            for service, data_list in frame['usage'].items():
-                frame_tenant = None
+            frame_tenant = None
+            for type_, points in frame.itertypes():
                 resources = []
-                for data in data_list:
-                    # This means we use a v1 storage backend
-                    if 'desc' in data.keys():
-                        desc = data['desc']
-                    else:
-                        desc = data['metadata'].copy()
-                        desc.update(data.get('groupby', {}))
-                    price = decimal.Decimal(str(data['rating']['price']))
+                for point in points:
                     resource = storage_models.RatedResource(
-                        service=service,
-                        desc=desc,
-                        volume=data['vol']['qty'],
-                        rating=price)
+                        service=type_,
+                        desc=point.desc,
+                        volume=point.qty,
+                        rating=point.price)
                     if frame_tenant is None:
-                        frame_tenant = data['scope_id']
+                        # NOTE(jferrieu): Since DataFrame/DataPoint
+                        # implementation patch we cannot guarantee
+                        # anymore that a DataFrame does contain a scope_id
+                        # therefore the __UNDEF__ default value has been
+                        # retained to maintain backward compatibility
+                        # if it would occur being absent
+                        frame_tenant = point.desc.get(scope_key, '__UNDEF__')
                     resources.append(resource)
                 dataframe = storage_models.DataFrame(
-                    begin=ck_utils.iso2dt(frame['period']['begin']),
-                    end=ck_utils.iso2dt(frame['period']['end']),
+                    begin=tzutils.local_to_utc(frame.start, naive=True),
+                    end=tzutils.local_to_utc(frame.end, naive=True),
                     tenant_id=frame_tenant,
                     resources=resources)
                 dataframes.append(dataframe)
