@@ -13,17 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
 import abc
 import fractions
 
 from oslo_config import cfg
+from oslo_log import log as logging
 import six
 from stevedore import driver
 from voluptuous import All
 from voluptuous import Any
 from voluptuous import Coerce
+from voluptuous import Error as VoluptuousError
 from voluptuous import In
 from voluptuous import Invalid
 from voluptuous import Length
@@ -31,9 +31,11 @@ from voluptuous import Optional
 from voluptuous import Required
 from voluptuous import Schema
 
-from cloudkitty import transformer
 from cloudkitty import utils as ck_utils
 
+
+LOG = logging.getLogger(__name__)
+
 collect_opts = [
     cfg.StrOpt('collector',
                default='gnocchi',
@@ -47,6 +49,10 @@ collect_opts = [
     cfg.StrOpt('metrics_conf',
                default='/etc/cloudkitty/metrics.yml',
                help='Metrology configuration file.'),
+    cfg.StrOpt('scope_key',
+               default='project_id',
+               help='Key defining a scope. project_id or domain_id for '
+               'OpenStack, but can be anything.'),
 ]
 
 CONF = cfg.CONF
@@ -94,15 +100,12 @@ METRIC_BASE_SCHEMA = {
 }
 
 
-def get_collector(transformers=None):
+def get_collector():
     metrics_conf = ck_utils.load_conf(CONF.collect.metrics_conf)
-    if not transformers:
-        transformers = transformer.get_transformers()
     collector_args = {
         'period': CONF.collect.period,
-        'transformers': transformers,
+        'conf': metrics_conf,
     }
-    collector_args.update({'conf': metrics_conf})
     return driver.DriverManager(
         COLLECTORS_NAMESPACE,
         CONF.collect.collector,
@@ -125,7 +128,6 @@ def get_metrics_based_collector_metadata():
     Results are based on enabled collector and metrics in CONF.
     """
     metrics_conf = ck_utils.load_conf(CONF.collect.metrics_conf)
-    transformers = transformer.get_transformers()
     collector = get_collector_without_invoke()
     metadata = {}
     if 'metrics' in metrics_conf:
@@ -133,23 +135,11 @@ def get_metrics_based_collector_metadata():
             alt_name = metric.get('alt_name', metric_name)
             metadata[alt_name] = collector.get_metadata(
                 metric_name,
-                transformers,
                 metrics_conf,
             )
     return metadata
 
 
-class TransformerDependencyError(Exception):
-    """Raised when a collector can't find a mandatory transformer."""
-
-    def __init__(self, collector, transformer):
-        super(TransformerDependencyError, self).__init__(
-            "Transformer '%s' not found, but required by %s" % (transformer,
-                                                                collector))
-        self.collector = collector
-        self.transformer = transformer
-
-
 class NoDataCollected(Exception):
     """Raised when the collection returned no data.
 
@@ -166,47 +156,40 @@ class NoDataCollected(Exception):
 @six.add_metaclass(abc.ABCMeta)
 class BaseCollector(object):
     collector_name = None
-    dependencies = ['CloudKittyFormatTransformer']
 
-    def __init__(self, transformers, **kwargs):
+    def __init__(self, **kwargs):
         try:
-            self.transformers = transformers
             self.period = kwargs['period']
             self.conf = self.check_configuration(kwargs['conf'])
         except KeyError as e:
-            raise ValueError("Missing argument (%s)" % e)
+            key_error_message = "Missing argument (%s)" % e
+            LOG.error(key_error_message, e)
+            raise ValueError(key_error_message)
+        except VoluptuousError as v:
+            LOG.error('Problem while checking configurations.', v)
+            raise v
 
-        self._check_transformers()
-        self.t_cloudkitty = self.transformers['CloudKittyFormatTransformer']
-
-    def _check_transformers(self):
-        """Check for transformer prerequisites
+    @staticmethod
+    def check_configuration(conf):
+        """Checks and validates metric configuration.
 
+        Collectors requiring extra parameters for metric collection
+        should implement this method, call the method of the parent class,
+        extend the ``extra_args`` key in ``METRIC_BASE_SCHEMA`` and validate
+        the metric configuration against the new schema.
         """
-        for dependency in self.dependencies:
-            if dependency not in self.transformers:
-                raise TransformerDependencyError(self.collector_name,
-                                                 dependency)
+        conf = Schema(CONF_BASE_SCHEMA)(conf)
+        metric_schema = Schema(METRIC_BASE_SCHEMA)
 
-    @staticmethod
-    def check_configuration(self, conf):
-        """Check metrics configuration
+        scope_key = CONF.collect.scope_key
 
-        """
-        return Schema(METRIC_BASE_SCHEMA)(conf)
+        output = {}
+        for metric_name, metric in conf['metrics'].items():
+            output[metric_name] = metric_schema(metric)
+            if scope_key not in output[metric_name]['groupby']:
+                output[metric_name]['groupby'].append(scope_key)
 
-    @staticmethod
-    def last_month():
-        month_start = ck_utils.get_month_start()
-        month_end = ck_utils.get_month_end()
-        start_ts = ck_utils.dt2ts(month_start)
-        end_ts = ck_utils.dt2ts(month_end)
-        return start_ts, end_ts
-
-    @staticmethod
-    def current_month():
-        month_start = ck_utils.get_month_start()
-        return ck_utils.dt2ts(month_start)
+        return output
 
     @classmethod
     def _res_to_func(cls, resource_name):
@@ -215,7 +198,7 @@ class BaseCollector(object):
         return trans_resource
 
     @classmethod
-    def get_metadata(cls, resource_name, transformers):
+    def get_metadata(cls, resource_name):
         """Return metadata about collected resource as a dict.
 
            Dict object should contain:
@@ -227,7 +210,25 @@ class BaseCollector(object):
     @abc.abstractmethod
     def fetch_all(self, metric_name, start, end,
                   project_id=None, q_filter=None):
-        pass
+        """Fetches information about a specific metric for a given period.
+
+        This method must respect the ``groupby`` and ``metadata`` arguments
+        provided in the metric conf at initialization.
+        (Available in ``self.conf['groupby']`` and ``self.conf['metadata']``).
+
+        Returns a list of cloudkitty.dataframe.DataPoint objects.
+
+        :param metric_name: Name of the metric to fetch
+        :type metric_name: str
+        :param start: start of the period
+        :type start: datetime.datetime
+        :param end: end of the period
+        :type end: datetime.datetime
+        :param project_id: ID of the scope for which data should be collected
+        :type project_id: str
+        :param q_filter: Optional filters
+        :type q_filter: dict
+        """
 
     def retrieve(self, metric_name, start, end,
                  project_id=None, q_filter=None):
@@ -241,11 +242,10 @@ class BaseCollector(object):
         )
 
         name = self.conf[metric_name].get('alt_name', metric_name)
-        if data:
-            data = self.t_cloudkitty.format_service(name, data)
         if not data:
             raise NoDataCollected(self.collector_name, name)
-        return data
+
+        return name, data
 
 
 def validate_conf(conf):
