@@ -13,8 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Luka Peschke
-#
 from keystoneauth1 import loading as ks_loading
 from keystoneclient.v3 import client as ks_client
 from monascaclient import client as mclient
@@ -27,7 +25,7 @@ from voluptuous import Required
 from voluptuous import Schema
 
 from cloudkitty import collector
-from cloudkitty import transformer
+from cloudkitty import dataframe
 from cloudkitty import utils as ck_utils
 
 
@@ -35,8 +33,8 @@ LOG = logging.getLogger(__name__)
 
 MONASCA_API_VERSION = '2_0'
 COLLECTOR_MONASCA_OPTS = 'collector_monasca'
-collector_monasca_opts = ks_loading.get_auth_common_conf_options()
-mcollector_opts = [
+
+collector_monasca_opts = [
     cfg.StrOpt(
         'interface',
         default='internal',
@@ -49,37 +47,32 @@ mcollector_opts = [
     ),
 ]
 
-cfg.CONF.register_opts(collector_monasca_opts, COLLECTOR_MONASCA_OPTS)
-cfg.CONF.register_opts(mcollector_opts, COLLECTOR_MONASCA_OPTS)
-ks_loading.register_session_conf_options(
-    cfg.CONF,
-    COLLECTOR_MONASCA_OPTS)
-ks_loading.register_auth_conf_options(
-    cfg.CONF,
-    COLLECTOR_MONASCA_OPTS)
 CONF = cfg.CONF
 
+CONF.register_opts(collector_monasca_opts, COLLECTOR_MONASCA_OPTS)
+ks_loading.register_auth_conf_options(CONF, COLLECTOR_MONASCA_OPTS)
+ks_loading.register_session_conf_options(CONF, COLLECTOR_MONASCA_OPTS)
+
 METRICS_CONF = ck_utils.load_conf(CONF.collect.metrics_conf)
 
 MONASCA_EXTRA_SCHEMA = {
-    Required('extra_args'): {
+    Required('extra_args', default={}): {
         # Key corresponding to the resource id in a metric's dimensions
         # Allows to adapt the resource identifier. Should not need to be
         # modified in a standard OpenStack installation
         Required('resource_key', default='resource_id'):
             All(str, Length(min=1)),
-        # This is needed to allow filtering on the project for the Openstack
-        # usecase. May be removed in following releases
-        Required('scope_key', default='project_id'): All(str, Length(min=1)),
         Required('aggregation_method', default='max'):
             In(['max', 'mean', 'min']),
+        # In case the metrics in Monasca do not belong to the project
+        # cloudkitty is identified in
+        Required('forced_project_id', default=''): str,
     },
 }
 
 
 class EndpointNotFound(Exception):
     """Exception raised if the Monasca endpoint is not found"""
-    pass
 
 
 class MonascaCollector(collector.BaseCollector):
@@ -87,20 +80,21 @@ class MonascaCollector(collector.BaseCollector):
 
     @staticmethod
     def check_configuration(conf):
-        """Check metrics configuration
-
-        """
-        conf = Schema(collector.CONF_BASE_SCHEMA)(conf)
+        conf = collector.BaseCollector.check_configuration(conf)
         metric_schema = Schema(collector.METRIC_BASE_SCHEMA).extend(
             MONASCA_EXTRA_SCHEMA)
 
-        output = dict()
-        for metric_name, metric in conf['metrics'].items():
-            output[metric_name] = metric_schema(metric)
+        output = {}
+        for metric_name, metric in conf.items():
+            met = output[metric_name] = metric_schema(metric)
+
+            if met['extra_args']['resource_key'] not in met['groupby']:
+                met['groupby'].append(met['extra_args']['resource_key'])
+
         return output
 
-    def __init__(self, transformers, **kwargs):
-        super(MonascaCollector, self).__init__(transformers, **kwargs)
+    def __init__(self, **kwargs):
+        super(MonascaCollector, self).__init__(**kwargs)
 
         self.auth = ks_loading.load_auth_from_conf_options(
             CONF,
@@ -134,7 +128,7 @@ class MonascaCollector(collector.BaseCollector):
                 return endpoint.url
         return None
 
-    def _get_metadata(self, metric_name, transformers, conf):
+    def _get_metadata(self, metric_name, conf):
         info = {}
         info['unit'] = conf['metrics'][metric_name]['unit']
 
@@ -146,18 +140,15 @@ class MonascaCollector(collector.BaseCollector):
     # NOTE(lukapeschke) if anyone sees a better way to do this,
     # please make a patch
     @classmethod
-    def get_metadata(cls, resource_type, transformers, conf):
-        args = {
-            'transformers': transformer.get_transformers(),
-            'period': conf['period']}
-        tmp = cls(**args)
-        return tmp._get_metadata(resource_type, transformers, conf)
+    def get_metadata(cls, resource_type, conf):
+        tmp = cls(period=conf['period'])
+        return tmp._get_metadata(resource_type, conf)
 
     def _get_dimensions(self, metric_name, project_id, q_filter):
-        extra_args = self.conf[metric_name]['extra_args']
         dimensions = {}
+        scope_key = CONF.collect.scope_key
         if project_id:
-            dimensions[extra_args['scope_key']] = project_id
+            dimensions[scope_key] = project_id
         if q_filter:
             dimensions.update(q_filter)
         return dimensions
@@ -178,26 +169,25 @@ class MonascaCollector(collector.BaseCollector):
 
         dimensions = self._get_dimensions(metric_name, project_id, q_filter)
         group_by = self.conf[metric_name]['groupby']
-        resource_key = self.conf[metric_name]['extra_args']['resource_key']
-        if resource_key not in group_by:
-            LOG.error('Resource key "{}" is not in group_by keys: "{}". '
-                      'Please adapt your configuration.'.format(
-                          resource_key, group_by))
-            raise collector.NoDataCollected(self.collector_name, metric_name)
 
         # NOTE(lpeschke): One aggregated measure per collect period
-        period = end - start
+        period = int((end - start).total_seconds())
 
         extra_args = self.conf[metric_name]['extra_args']
+        kwargs = {}
+        if extra_args['forced_project_id']:
+            kwargs['tenant_id'] = extra_args['forced_project_id']
+
         return self._conn.metrics.list_statistics(
             name=metric_name,
             merge_metrics=True,
             dimensions=dimensions,
-            start_time=ck_utils.ts2dt(start),
-            end_time=ck_utils.ts2dt(end),
+            start_time=start,
+            end_time=end,
             period=period,
             statistics=extra_args['aggregation_method'],
-            group_by=group_by)
+            group_by=group_by,
+            **kwargs)
 
     def _fetch_metrics(self, metric_name, start, end,
                        project_id=None, q_filter=None):
@@ -216,8 +206,8 @@ class MonascaCollector(collector.BaseCollector):
         metrics = self._conn.metrics.list(
             name=metric_name,
             dimensions=dimensions,
-            start_time=ck_utils.ts2dt(start),
-            end_time=ck_utils.ts2dt(end),
+            start_time=start,
+            end_time=end,
         )
 
         resource_key = self.conf[metric_name]['extra_args']['resource_key']
@@ -273,11 +263,11 @@ class MonascaCollector(collector.BaseCollector):
             if len(d['statistics']):
                 metadata, groupby, qty = self._format_data(
                     met, d, resources_info)
-                data = self.t_cloudkitty.format_item(
+                formated_resources.append(dataframe.DataPoint(
+                    met['unit'],
+                    qty,
+                    0,
                     groupby,
                     metadata,
-                    met['unit'],
-                    qty=qty,
-                )
-                formated_resources.append(data)
+                ))
         return formated_resources
