@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2018 Objectif Libre
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,102 +12,105 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Martin CAMEY
-#
 from decimal import Decimal
 from decimal import localcontext
 from decimal import ROUND_HALF_UP
 
 from oslo_config import cfg
 from oslo_log import log
-import requests
-from voluptuous import All
-from voluptuous import Length
+from voluptuous import In
 from voluptuous import Required
 from voluptuous import Schema
 
 from cloudkitty import collector
+from cloudkitty.collector.exceptions import CollectError
+from cloudkitty.common.prometheus_client import PrometheusClient
+from cloudkitty.common.prometheus_client import PrometheusResponseError
+from cloudkitty import dataframe
+from cloudkitty import tzutils
 from cloudkitty import utils as ck_utils
 
 
 LOG = log.getLogger(__name__)
 
-PROMETHEUS_COLLECTOR_OPTS = 'prometheus_collector'
-pcollector_collector_opts = [
+PROMETHEUS_COLLECTOR_OPTS = 'collector_prometheus'
+collector_prometheus_opts = [
     cfg.StrOpt(
         'prometheus_url',
         default='',
         help='Prometheus service URL',
     ),
+    cfg.StrOpt(
+        'prometheus_user',
+        help='Prometheus user (for basic auth only)',
+    ),
+    cfg.StrOpt(
+        'prometheus_password',
+        help='Prometheus user password (for basic auth only)',
+        secret=True,
+    ),
+    cfg.StrOpt(
+        'cafile',
+        help='Custom certificate authority file path',
+    ),
+    cfg.BoolOpt(
+        'insecure',
+        default=False,
+        help='Explicitly trust untrusted HTTPS responses',
+    ),
 ]
-cfg.CONF.register_opts(pcollector_collector_opts, PROMETHEUS_COLLECTOR_OPTS)
+cfg.CONF.register_opts(collector_prometheus_opts, PROMETHEUS_COLLECTOR_OPTS)
 
 CONF = cfg.CONF
 
 PROMETHEUS_EXTRA_SCHEMA = {
-    Required('extra_args'): {
-        Required('query'): All(str, Length(min=1)),
+    Required('extra_args', default={}): {
+        Required('aggregation_method', default='max'):
+            In([
+                'avg', 'count', 'max',
+                'min', 'stddev', 'stdvar',
+                'sum'
+            ]),
     }
 }
 
 
-class PrometheusClient(object):
-    @classmethod
-    def build_query(cls, source, query, start, end, period, metric_name):
-        """Build PromQL instant queries."""
-        start = ck_utils.iso8601_from_timestamp(start)
-        end = ck_utils.iso8601_from_timestamp(end)
-
-        if '$period' in query:
-            try:
-                query = ck_utils.template_str_substitute(
-                    query, {'period': str(period) + 's'},
-                )
-            except (KeyError, ValueError):
-                raise collector.NoDataCollected(
-                    collector.collector_name,
-                    metric_name
-                )
-
-        # Due to the design of Cloudkitty, only instant queries are supported.
-        # In that case 'time' equals 'end' and
-        # the window time is reprezented by the period.
-        return source + '/query?query=' + query + '&time=' + end
-
-    @classmethod
-    def get_data(cls, source, query, start, end, period, metric_name):
-        url = cls.build_query(
-            source,
-            query,
-            start,
-            end,
-            period,
-            metric_name,
-        )
-
-        return requests.get(url).json()
-
-
 class PrometheusCollector(collector.BaseCollector):
     collector_name = 'prometheus'
 
-    def __init__(self, transformers, **kwargs):
-        super(PrometheusCollector, self).__init__(transformers, **kwargs)
+    def __init__(self, **kwargs):
+        super(PrometheusCollector, self).__init__(**kwargs)
+        url = CONF.collector_prometheus.prometheus_url
+
+        user = CONF.collector_prometheus.prometheus_user
+        password = CONF.collector_prometheus.prometheus_password
+
+        verify = True
+        if CONF.collector_prometheus.cafile:
+            verify = CONF.collector_prometheus.cafile
+        elif CONF.collector_prometheus.insecure:
+            verify = False
+
+        self._conn = PrometheusClient(
+            url,
+            auth=(user, password) if user and password else None,
+            verify=verify,
+        )
 
     @staticmethod
     def check_configuration(conf):
-        """Check metrics configuration."""
-        conf = Schema(collector.CONF_BASE_SCHEMA)(conf)
+        conf = collector.BaseCollector.check_configuration(conf)
         metric_schema = Schema(collector.METRIC_BASE_SCHEMA).extend(
             PROMETHEUS_EXTRA_SCHEMA,
         )
 
         output = {}
-        for metric_name, metric in conf['metrics'].items():
+        for metric_name, metric in conf.items():
             output[metric_name] = metric_schema(metric)
+
         return output
 
-    def _format_data(self, metric_name, project_id, start, end, data):
+    def _format_data(self, metric_name, scope_key, scope_id, start, end, data):
         """Formats Prometheus data format to Cloudkitty data format.
 
         Returns metadata, groupby, qty
@@ -117,7 +119,7 @@ class PrometheusCollector(collector.BaseCollector):
         for meta in self.conf[metric_name]['metadata']:
             metadata[meta] = data['metric'][meta]
 
-        groupby = {}
+        groupby = {scope_key: scope_id}
         for meta in self.conf[metric_name]['groupby']:
             groupby[meta] = data['metric'].get(meta, '')
 
@@ -133,46 +135,55 @@ class PrometheusCollector(collector.BaseCollector):
 
         return metadata, groupby, qty
 
-    def fetch_all(self, metric_name, start, end, project_id, q_filter=None):
+    def fetch_all(self, metric_name, start, end, scope_id, q_filter=None):
         """Returns metrics to be valorized."""
-        # NOTE(mc): Remove potential trailing '/' to avoid
-        # url building problems
-        url = CONF.prometheus_collector.prometheus_url
-        if url.endswith('/'):
-            url = url[:-1]
+        scope_key = CONF.collect.scope_key
+        method = self.conf[metric_name]['extra_args']['aggregation_method']
+        groupby = self.conf[metric_name].get('groupby', [])
+        metadata = self.conf[metric_name].get('metadata', [])
+        period = tzutils.diff_seconds(end, start)
+        time = end
 
-        res = PrometheusClient.get_data(
-            url,
-            self.conf[metric_name]['extra_args']['query'],
-            start,
-            end,
-            self.period,
+        query = '{0}({0}_over_time({1}{{{2}="{3}"}}[{4}s])) by ({5})'.format(
+            method,
             metric_name,
+            scope_key,
+            scope_id,
+            period,
+            ', '.join(groupby + metadata),
         )
 
+        try:
+            res = self._conn.get_instant(
+                query,
+                time.isoformat(),
+            )
+        except PrometheusResponseError as e:
+            raise CollectError(*e.args)
+
         # If the query returns an empty dataset,
-        # raise a NoDataCollected exception.
+        # return an empty list
         if not res['data']['result']:
-            raise collector.NoDataCollected(self.collector_name, metric_name)
+            return []
 
         formatted_resources = []
 
         for item in res['data']['result']:
             metadata, groupby, qty = self._format_data(
                 metric_name,
-                project_id,
+                scope_key,
+                scope_id,
                 start,
                 end,
                 item,
             )
 
-            item = self.t_cloudkitty.format_item(
+            formatted_resources.append(dataframe.DataPoint(
+                self.conf[metric_name]['unit'],
+                qty,
+                0,
                 groupby,
                 metadata,
-                self.conf[metric_name]['unit'],
-                qty=qty,
-            )
-
-            formatted_resources.append(item)
+            ))
 
         return formatted_resources
