@@ -13,12 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
+from datetime import timedelta
 import decimal
+import functools
+import hashlib
+import multiprocessing
 import random
+import sys
+import time
 
-import eventlet
+import cotyledon
+import futurist
+from futurist import waiters
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -29,25 +35,40 @@ from tooz import coordination
 
 from cloudkitty import collector
 from cloudkitty import config  # noqa
+from cloudkitty import dataframe
 from cloudkitty import extension_manager
 from cloudkitty import messaging
 from cloudkitty import storage
 from cloudkitty import storage_state as state
-from cloudkitty import transformer
+from cloudkitty import tzutils
 from cloudkitty import utils as ck_utils
 
-eventlet.monkey_patch()
 
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
 
 orchestrator_opts = [
-    cfg.StrOpt('coordination_url',
-               secret=True,
-               help='Coordination driver URL',
-               default='file:///var/lib/cloudkitty/locks'),
+    cfg.StrOpt(
+        'coordination_url',
+        secret=True,
+        help='Coordination driver URL',
+        default='file:///var/lib/cloudkitty/locks'),
+    cfg.IntOpt(
+        'max_workers',
+        default=multiprocessing.cpu_count(),
+        min=1,
+        help='Max nb of workers to run. Defaults to the nb of available CPUs'),
+    cfg.IntOpt('max_threads',
+               # NOTE(peschk_l): This is the futurist default
+               default=multiprocessing.cpu_count() * 5,
+               min=1,
+               deprecated_name='max_greenthreads',
+               advanced=True,
+               help='Maximal number of threads to use per worker. Defaults to '
+               '5 times the nb of available CPUs'),
 ]
+
 CONF.register_opts(orchestrator_opts, group='orchestrator')
 
 CONF.import_opt('backend', 'cloudkitty.fetcher', 'fetcher')
@@ -58,9 +79,19 @@ COLLECTORS_NAMESPACE = 'cloudkitty.collector.backends'
 STORAGES_NAMESPACE = 'cloudkitty.storage.backends'
 
 
+def get_lock(coord, tenant_id):
+    name = hashlib.sha256(
+        ("cloudkitty-"
+         + str(tenant_id + '-')
+         + str(CONF.collect.collector + '-')
+         + str(CONF.fetcher.backend + '-')
+         + str(CONF.collect.scope_key)).encode('ascii')).hexdigest()
+    return name, coord.get_lock(name.encode('ascii'))
+
+
 class RatingEndpoint(object):
     target = oslo_messaging.Target(namespace='rating',
-                                   version='1.1')
+                                   version='1.0')
 
     def __init__(self, orchestrator):
         self._global_reload = False
@@ -115,6 +146,54 @@ class RatingEndpoint(object):
                 self._pending_reload.remove(name)
 
 
+class ScopeEndpoint(object):
+    target = oslo_messaging.Target(version='1.0')
+
+    def __init__(self):
+        self._coord = coordination.get_coordinator(
+            CONF.orchestrator.coordination_url,
+            uuidutils.generate_uuid().encode('ascii'))
+        self._state = state.StateManager()
+        self._storage = storage.get_storage()
+        self._coord.start(start_heart=True)
+
+    def reset_state(self, ctxt, res_data):
+        LOG.info('Received state reset command. {}'.format(res_data))
+        random.shuffle(res_data['scopes'])
+        for scope in res_data['scopes']:
+            lock_name, lock = get_lock(self._coord, scope['scope_id'])
+            LOG.debug(
+                '[ScopeEndpoint] Trying to acquire lock "{}" ...'.format(
+                    lock_name,
+                )
+            )
+            if lock.acquire(blocking=True):
+                LOG.debug(
+                    '[ScopeEndpoint] Acquired lock "{}".'.format(
+                        lock_name,
+                    )
+                )
+                state_dt = tzutils.dt_from_iso(res_data['state'])
+                try:
+                    self._storage.delete(begin=state_dt, end=None, filters={
+                        scope['scope_key']: scope['scope_id'],
+                    })
+                    self._state.set_state(
+                        scope['scope_id'],
+                        state_dt,
+                        fetcher=scope['fetcher'],
+                        collector=scope['collector'],
+                        scope_key=scope['scope_key'],
+                    )
+                finally:
+                    lock.release()
+                    LOG.debug(
+                        '[ScopeEndpoint] Released lock "{}" .'.format(
+                            lock_name,
+                        )
+                    )
+
+
 class BaseWorker(object):
     def __init__(self, tenant_id=None):
         self._tenant_id = tenant_id
@@ -151,108 +230,134 @@ class APIWorker(BaseWorker):
         return price
 
 
+def _check_state(obj, period, tenant_id):
+    timestamp = obj._state.get_state(tenant_id)
+    return ck_utils.check_time_state(timestamp,
+                                     period,
+                                     CONF.collect.wait_periods)
+
+
 class Worker(BaseWorker):
-    def __init__(self, collector, storage, tenant_id):
+    def __init__(self, collector, storage, tenant_id, worker_id):
         self._collector = collector
         self._storage = storage
         self._period = CONF.collect.period
         self._wait_time = CONF.collect.wait_periods * self._period
         self._tenant_id = tenant_id
+        self._worker_id = worker_id
+        self._log_prefix = '[scope: {scope}, worker: {worker}] '.format(
+            scope=self._tenant_id, worker=self._worker_id)
         self._conf = ck_utils.load_conf(CONF.collect.metrics_conf)
         self._state = state.StateManager()
+        self._check_state = functools.partial(
+            _check_state, self, self._period, self._tenant_id)
 
         super(Worker, self).__init__(self._tenant_id)
 
     def _collect(self, metric, start_timestamp):
-        next_timestamp = start_timestamp + self._period
+        next_timestamp = tzutils.add_delta(
+            start_timestamp, timedelta(seconds=self._period))
 
-        raw_data = self._collector.retrieve(
+        name, data = self._collector.retrieve(
             metric,
             start_timestamp,
             next_timestamp,
             self._tenant_id,
         )
+        if not data:
+            raise collector.NoDataCollected
 
-        if raw_data:
-            return [{'period': {'begin': start_timestamp,
-                                'end': next_timestamp},
-                     'usage': raw_data}]
+        return name, data
 
-    def check_state(self):
-        timestamp = self._state.get_state(self._tenant_id)
-        return ck_utils.check_time_state(timestamp,
-                                         self._period,
-                                         CONF.collect.wait_periods)
+    def _do_collection(self, metrics, timestamp):
+
+        def _get_result(metric):
+            try:
+                return self._collect(metric, timestamp)
+            except collector.NoDataCollected:
+                LOG.info(
+                    self._log_prefix + 'No data collected '
+                    'for metric {metric} at timestamp {ts}'.format(
+                        metric=metric, ts=timestamp))
+                return metric, None
+            except Exception as e:
+                LOG.exception(
+                    self._log_prefix + 'Error while collecting'
+                    ' metric {metric} at timestamp {ts}: {e}. Exiting.'.format(
+                        metric=metric, ts=timestamp, e=e))
+                # FIXME(peschk_l): here we just exit, and the
+                # collection will be retried during the next collect
+                # cycle. In the future, we should implement a retrying
+                # system in workers
+                sys.exit(1)
+
+        with futurist.ThreadPoolExecutor(
+                max_workers=CONF.orchestrator.max_threads) as tpool:
+            futs = [tpool.submit(_get_result, metric) for metric in metrics]
+            LOG.debug(self._log_prefix +
+                      'Collecting {} metrics.'.format(len(metrics)))
+            results = [r.result() for r in waiters.wait_for_all(futs).done]
+            LOG.debug(self._log_prefix + 'Collecting {} metrics took {}s '
+                      'total, with {}s average'.format(
+                          tpool.statistics.executed,
+                          tpool.statistics.runtime,
+                          tpool.statistics.average_runtime))
+        return dict(filter(lambda x: x[1] is not None, results))
 
     def run(self):
         while True:
-            timestamp = self.check_state()
+            timestamp = self._check_state()
             if not timestamp:
                 break
 
             metrics = list(self._conf['metrics'].keys())
 
-            storage_data = []
-            for metric in metrics:
-                try:
-                    try:
-                        data = self._collect(metric, timestamp)
-                    except collector.NoDataCollected:
-                        raise
-                    except Exception as e:
-                        LOG.warning(
-                            'Error while collecting metric '
-                            '%(metric)s: %(error)s',
-                            {'metric': metric, 'error': e})
-                        raise collector.NoDataCollected('', metric)
-                except collector.NoDataCollected:
-                    LOG.info(
-                        'No data collected for metric {} '
-                        'at timestamp {}'.format(
-                            metric, ck_utils.ts2dt(timestamp))
-                    )
-                else:
-                    # Rating
-                    for processor in self._processors:
-                        processor.obj.process(data)
-                    # Writing
-                    if isinstance(data, list):
-                        storage_data += data
-                    else:
-                        storage_data.append(data)
+            # Collection
+            usage_data = self._do_collection(metrics, timestamp)
 
-            # We're getting a full period so we directly commit
-            self._storage.push(storage_data, self._tenant_id)
+            frame = dataframe.DataFrame(
+                start=timestamp,
+                end=tzutils.add_delta(timestamp,
+                                      timedelta(seconds=self._period)),
+                usage=usage_data,
+            )
+            # Rating
+            for processor in self._processors:
+                frame = processor.obj.process(frame)
+
+            # Writing
+            self._storage.push([frame], self._tenant_id)
             self._state.set_state(self._tenant_id, timestamp)
 
 
-class Orchestrator(object):
-    def __init__(self):
+class Orchestrator(cotyledon.Service):
+    def __init__(self, worker_id):
+        self._worker_id = worker_id
+        super(Orchestrator, self).__init__(self._worker_id)
+
         self.fetcher = driver.DriverManager(
             FETCHERS_NAMESPACE,
             CONF.fetcher.backend,
             invoke_on_load=True,
         ).driver
 
-        transformers = transformer.get_transformers()
-        self.collector = collector.get_collector(transformers)
+        self.collector = collector.get_collector()
         self.storage = storage.get_storage()
         self._state = state.StateManager()
 
         # RPC
         self.server = None
         self._rating_endpoint = RatingEndpoint(self)
+        self._scope_endpoint = ScopeEndpoint()
         self._init_messaging()
 
         # DLM
         self.coord = coordination.get_coordinator(
             CONF.orchestrator.coordination_url,
             uuidutils.generate_uuid().encode('ascii'))
-        self.coord.start()
-
-    def _lock(self, tenant_id):
-        lock_name = b"cloudkitty-" + str(tenant_id).encode('ascii')
-        return self.coord.get_lock(lock_name)
+        self.coord.start(start_heart=True)
+        self._check_state = functools.partial(
+            _check_state, self, CONF.collect.period)
 
     def _init_messaging(self):
         target = oslo_messaging.Target(topic='cloudkitty',
@@ -260,16 +365,11 @@ class Orchestrator(object):
                                        version='1.0')
         endpoints = [
             self._rating_endpoint,
+            self._scope_endpoint,
         ]
         self.server = messaging.get_server(target, endpoints)
         self.server.start()
 
-    def _check_state(self, tenant_id):
-        timestamp = self._state.get_state(tenant_id)
-        return ck_utils.check_time_state(timestamp,
-                                         CONF.collect.period,
-                                         CONF.collect.wait_periods)
-
     def process_messages(self):
         # TODO(sheeprine): Code kept to handle threading and asynchronous
         # reloading
@@ -277,33 +377,50 @@ class Orchestrator(object):
         # pending_states = self._rating_endpoint.get_module_state()
         pass
 
-    def process(self):
+    def run(self):
+        LOG.debug('Started worker {}.'.format(self._worker_id))
         while True:
             self.tenants = self.fetcher.get_tenants()
             random.shuffle(self.tenants)
-            LOG.info('Tenants loaded for fetcher %s', self.fetcher.name)
+            LOG.info('[Worker: {w}] Tenants loaded for fetcher {f}'.format(
+                w=self._worker_id, f=self.fetcher.name))
 
             for tenant_id in self.tenants:
 
-                lock = self._lock(tenant_id)
+                lock_name, lock = get_lock(self.coord, tenant_id)
+                LOG.debug(
+                    '[Worker: {w}] Trying to acquire lock "{l}" ...'.format(
+                        w=self._worker_id, l=lock_name)
+                )
                 if lock.acquire(blocking=False):
+                    LOG.debug(
+                        '[Worker: {w}] Acquired lock "{l}" ...'.format(
+                            w=self._worker_id, l=lock_name)
+                    )
                     state = self._check_state(tenant_id)
                     if state:
                         worker = Worker(
                             self.collector,
                             self.storage,
                             tenant_id,
+                            self._worker_id,
                         )
                         worker.run()
 
                     lock.release()
 
-                self.coord.heartbeat()
-                # NOTE(sheeprine): Slow down looping if all tenants are
-                # being processed
-                eventlet.sleep(1)
             # FIXME(sheeprine): We may cause a drift here
-            eventlet.sleep(CONF.collect.period)
+            time.sleep(CONF.collect.period)
 
     def terminate(self):
+        LOG.debug('Terminating worker {}...'.format(self._worker_id))
         self.coord.stop()
+        LOG.debug('Terminated worker {}.'.format(self._worker_id))
+
+
+class OrchestratorServiceManager(cotyledon.ServiceManager):
+
+    def __init__(self):
+        super(OrchestratorServiceManager, self).__init__()
+        self.service_id = self.add(Orchestrator,
+                                   workers=CONF.orchestrator.max_workers)
