@@ -13,23 +13,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
+import functools
+
 from oslo_config import cfg
 from oslo_log import log as logging
 from stevedore import driver
 
+from cloudkitty import dataframe
 from cloudkitty.storage import v2 as storage_v2
-
+from cloudkitty import tzutils
 
 LOG = logging.getLogger(__name__)
 
 
 storage_opts = [
     cfg.StrOpt('backend',
-               default='sqlalchemy',
+               default='influxdb',
                help='Name of the storage backend driver.'),
     cfg.IntOpt('version',
                min=1, max=2,
-               default=1,
+               default=2,
                help='Storage version to use.'),
 ]
 
@@ -63,13 +66,30 @@ class V1StorageAdapter(storage_v2.BaseStorage):
     def __init__(self, storage_args, storage_namespace, backend=None):
         self.storage = _get_storage_instance(
             storage_args, storage_namespace, backend=backend)
+        self._localize_dataframes = functools.partial(
+            self.__update_frames_timestamps, tzutils.utc_to_local)
+        self._make_dataframes_naive = functools.partial(
+            self.__update_frames_timestamps, tzutils.local_to_utc, naive=True)
 
     def init(self):
         return self.storage.init()
 
-    def push(self, dataframes, scope_id):
+    @staticmethod
+    def __update_frames_timestamps(func, frames, **kwargs):
+        for frame in frames:
+            start = frame.start
+            end = frame.end
+            if start:
+                frame.start = func(start, **kwargs)
+            if end:
+                frame.end = func(end, **kwargs)
+
+    def push(self, dataframes, scope_id=None):
         if dataframes:
-            self.storage.append(dataframes, scope_id)
+            self._make_dataframes_naive(dataframes)
+            self.storage.append(
+                [d.as_dict(mutable=True, legacy=True) for d in dataframes],
+                scope_id)
             self.storage.commit(scope_id)
 
     @staticmethod
@@ -79,32 +99,40 @@ class V1StorageAdapter(storage_v2.BaseStorage):
         return metric_types
 
     def retrieve(self, begin=None, end=None,
-                 filters=None, group_filters=None,
+                 filters=None,
                  metric_types=None,
                  offset=0, limit=100, paginate=True):
-        tenant_id = group_filters.get('project_id') if group_filters else None
+        tenant_id = filters.get('project_id') if filters else None
         metric_types = self._check_metric_types(metric_types)
         frames = self.storage.get_time_frame(
-            begin, end,
+            tzutils.local_to_utc(begin, naive=True) if begin else None,
+            tzutils.local_to_utc(end, naive=True) if end else None,
             res_type=metric_types,
             tenant_id=tenant_id)
-
-        for frame in frames:
-            for _, data_list in frame['usage'].items():
-                for data in data_list:
-                    data['scope_id'] = (data.get('project_id')
-                                        or data.get('tenant_id'))
-
+        frames = [dataframe.DataFrame.from_dict(frame, legacy=True)
+                  for frame in frames]
+        self._localize_dataframes(frames)
         return {
             'total': len(frames),
             'dataframes': frames,
         }
 
+    @staticmethod
+    def _localize_total(iterable):
+        for elem in iterable:
+            begin = elem['begin']
+            end = elem['end']
+            if begin:
+                elem['begin'] = tzutils.utc_to_local(begin)
+            if end:
+                elem['end'] = tzutils.utc_to_local(end)
+
     def total(self, groupby=None,
               begin=None, end=None,
               metric_types=None,
-              filters=None, group_filters=None):
-        tenant_id = group_filters.get('project_id') if group_filters else None
+              filters=None,
+              offset=0, limit=100, paginate=True):
+        tenant_id = filters.get('project_id') if filters else None
 
         storage_gby = []
         if groupby:
@@ -116,7 +144,8 @@ class V1StorageAdapter(storage_v2.BaseStorage):
         storage_gby = ','.join(storage_gby) if storage_gby else None
         metric_types = self._check_metric_types(metric_types)
         total = self.storage.get_total(
-            begin, end,
+            tzutils.local_to_utc(begin, naive=True),
+            tzutils.local_to_utc(end, naive=True),
             tenant_id=tenant_id,
             service=metric_types,
             groupby=storage_gby)
@@ -130,7 +159,11 @@ class V1StorageAdapter(storage_v2.BaseStorage):
                 t['type'] = t.get('res_type')
             else:
                 t['type'] = None
-        return total
+        self._localize_total(total)
+        return {
+            'total': len(total),
+            'results': total,
+        }
 
     def get_tenants(self, begin, end):
         tenants = self.storage.get_tenants(begin, end)
@@ -139,6 +172,9 @@ class V1StorageAdapter(storage_v2.BaseStorage):
     def get_state(self, tenant_id=None):
         return self.storage.get_state(tenant_id)
 
+    def delete(self, begin=None, end=None, filters=None):
+        LOG.warning('Calling unsupported "delete" method on v1 storage.')
+
 
 def get_storage(**kwargs):
     storage_args = {
@@ -149,8 +185,8 @@ def get_storage(**kwargs):
 
     version = kwargs.pop('version', None) or cfg.CONF.storage.version
     if int(version) > 1:
-        LOG.warning('V2 Storage is not considered stable and should not be '
-                    'used in production')
+        LOG.warning('V2 Storage is in beta. Its API is considered stable but '
+                    'its implementation may still evolve.')
     storage_namespace = 'cloudkitty.storage.v{}.backends'.format(version)
 
     if version == 1:
