@@ -13,11 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Luka Peschke
-#
 import datetime
 import decimal
-import json
 
 from gnocchiclient import client as gclient
 from gnocchiclient import exceptions as gexceptions
@@ -28,6 +25,7 @@ from oslo_utils import uuidutils
 import six
 
 from cloudkitty.collector import validate_conf
+from cloudkitty import json_utils as json
 from cloudkitty.storage.v1.hybrid.backends import BaseHybridBackend
 import cloudkitty.utils as ck_utils
 
@@ -67,14 +65,6 @@ RESOURCE_TYPE_NAME_ROOT = 'rating_service_'
 METADATA_NAME_ROOT = 'ckmeta_'
 
 
-class DecimalJSONEncoder(json.JSONEncoder):
-    """Wrapper class to handle decimal.Decimal objects in json.dumps()."""
-    def default(self, obj):
-        if isinstance(obj, decimal.Decimal):
-            return float(obj)
-        return super(DecimalJSONEncoder, self).default(obj)
-
-
 class UnknownResourceType(Exception):
     """Exception raised when an unknown resource type is encountered"""
 
@@ -408,7 +398,7 @@ class GnocchiStorage(BaseHybridBackend):
         price_dict = {'price': float(price)}
 
         # Getting vol
-        if isinstance(res_type_info['qty_metric'], six.text_type):
+        if isinstance(res_type_info['qty_metric'], six.string_types):
             try:
                 qty = self._conn.metric.get_measures(
                     resource['metrics'][res_type_info['qty_metric']],
