@@ -13,13 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
 import abc
+import collections
+import datetime
 import decimal
 import os
-from unittest.case import SkipTest
 
+from dateutil import tz
 from gabbi import fixture
 import mock
 from oslo_config import cfg
@@ -37,19 +37,22 @@ import wsmeext.pecan as wsme_pecan
 
 from cloudkitty.api import app
 from cloudkitty.api import middleware
+from cloudkitty import dataframe
 from cloudkitty import db
 from cloudkitty.db import api as ck_db_api
 from cloudkitty import messaging
 from cloudkitty import rating
 from cloudkitty import storage
 from cloudkitty.storage.v1.sqlalchemy import models
+from cloudkitty import storage_state
 from cloudkitty import tests
-from cloudkitty.tests import test_utils
-from cloudkitty.tests.utils import is_functional_test
+from cloudkitty.tests.storage.v2 import influx_utils
+from cloudkitty.tests import utils as test_utils
+from cloudkitty import tzutils
 from cloudkitty import utils as ck_utils
 
 
-INITIAL_TIMESTAMP = 1420070400
+INITIAL_DT = datetime.datetime(2015, 1, 1, tzinfo=tz.UTC)
 
 
 class UUIDFixture(fixture.GabbiFixture):
@@ -86,10 +89,9 @@ class BaseExtensionFixture(fixture.GabbiFixture):
         self.patch.return_value = fake_mgr
 
     def stop_fixture(self):
-        if not is_functional_test():
-            self.patch.assert_called_with(
-                self.namespace,
-                **self.assert_args)
+        self.patch.assert_called_with(
+            self.namespace,
+            **self.assert_args)
         self.mock.stop()
 
 
@@ -218,6 +220,14 @@ class ConfigFixture(fixture.GabbiFixture):
         db.get_engine().dispose()
 
 
+class ConfigFixtureStorageV2(ConfigFixture):
+
+    def start_fixture(self):
+        super(ConfigFixtureStorageV2, self).start_fixture()
+        self.conf.set_override('backend', 'influxdb', 'storage')
+        self.conf.set_override('version', '2', 'storage')
+
+
 class ConfigFixtureKeystoneAuth(ConfigFixture):
     auth_strategy = 'keystone'
 
@@ -265,6 +275,16 @@ class BaseFakeRPC(fixture.GabbiFixture):
         self.server.stop()
 
 
+class ScopeStateResetFakeRPC(BaseFakeRPC):
+    class FakeRPCEndpoint(object):
+        target = oslo_messaging.Target(version='1.0')
+
+        def reset_state(self, ctxt, res_data):
+            pass
+
+    endpoint = FakeRPCEndpoint
+
+
 class QuoteFakeRPC(BaseFakeRPC):
     class FakeRPCEndpoint(object):
         target = oslo_messaging.Target(namespace='rating',
@@ -277,36 +297,33 @@ class QuoteFakeRPC(BaseFakeRPC):
 
 
 class BaseStorageDataFixture(fixture.GabbiFixture):
-    def create_fake_data(self, begin, end):
-        data = [{
-            "period": {
-                "begin": begin,
-                "end": end},
-            "usage": {
-                "cpu": [
-                    {
-                        "desc": {
-                            "dummy": True,
-                            "fake_meta": 1.0},
-                        "vol": {
-                            "qty": 1,
-                            "unit": "nothing"},
-                        "rating": {
-                            "price": decimal.Decimal('1.337')}}]}}, {
-            "period": {
-                "begin": begin,
-                "end": end},
-            "usage": {
-                "image.size": [
-                    {
-                        "desc": {
-                            "dummy": True,
-                            "fake_meta": 1.0},
-                        "vol": {
-                            "qty": 1,
-                            "unit": "nothing"},
-                        "rating": {
-                            "price": decimal.Decimal('0.121')}}]}}]
+    def create_fake_data(self, begin, end, project_id):
+
+        cpu_point = dataframe.DataPoint(
+            unit="nothing",
+            qty=1,
+            groupby={"fake_meta": 1.0, "project_id": project_id},
+            metadata={"dummy": True},
+            price=decimal.Decimal('1.337'),
+        )
+        image_point = dataframe.DataPoint(
+            unit="nothing",
+            qty=1,
+            groupby={"fake_meta": 1.0, "project_id": project_id},
+            metadata={"dummy": True},
+            price=decimal.Decimal('0.121'),
+        )
+        data = [
+            dataframe.DataFrame(
+                start=begin, end=end,
+                usage=collections.OrderedDict({"cpu": [cpu_point, cpu_point]}),
+            ),
+            dataframe.DataFrame(
+                start=begin, end=end,
+                usage=collections.OrderedDict(
+                    {"image.size": [image_point, image_point]}),
+            ),
+        ]
         return data
 
     def start_fixture(self):
@@ -334,32 +351,65 @@ class BaseStorageDataFixture(fixture.GabbiFixture):
 class StorageDataFixture(BaseStorageDataFixture):
     def initialize_data(self):
         nodata_duration = (24 * 3 + 12) * 3600
+        hour_delta = datetime.timedelta(seconds=3600)
         tenant_list = ['8f82cc70-e50c-466e-8624-24bdea811375',
                        '7606a24a-b8ad-4ae0-be6c-3d7a41334a2e']
-        data_ts = INITIAL_TIMESTAMP + nodata_duration + 3600
-        data_duration = (24 * 2 + 8) * 3600
-        for i in range(data_ts,
-                       data_ts + data_duration,
-                       3600):
-            data = self.create_fake_data(i, i + 3600)
+        data_dt = INITIAL_DT + datetime.timedelta(
+            seconds=nodata_duration + 3600)
+        data_duration = datetime.timedelta(seconds=(24 * 2 + 8) * 3600)
+
+        iter_dt = data_dt
+        while iter_dt < data_dt + data_duration:
+            data = self.create_fake_data(
+                iter_dt, iter_dt + hour_delta, tenant_list[0])
             self.storage.push(data, tenant_list[0])
-        half_duration = int(data_duration / 2)
-        for i in range(data_ts,
-                       data_ts + half_duration,
-                       3600):
-            data = self.create_fake_data(i, i + 3600)
+            iter_dt += hour_delta
+
+        iter_dt = data_dt
+        while iter_dt < data_dt + data_duration / 2:
+            data = self.create_fake_data(
+                iter_dt, iter_dt + hour_delta, tenant_list[1])
             self.storage.push(data, tenant_list[1])
+            iter_dt += hour_delta
 
 
 class NowStorageDataFixture(BaseStorageDataFixture):
     def initialize_data(self):
-        begin = ck_utils.get_month_start_timestamp()
-        for i in range(begin,
-                       begin + 3600 * 12,
-                       3600):
-            data = self.create_fake_data(i, i + 3600)
-            self.storage.push(data,
-                              '3d9a1b33-482f-42fd-aef9-b575a3da9369')
+        dt = tzutils.get_month_start(naive=True).replace(tzinfo=tz.UTC)
+        hour_delta = datetime.timedelta(seconds=3600)
+        limit = dt + hour_delta * 12
+        while dt < limit:
+            project_id = '3d9a1b33-482f-42fd-aef9-b575a3da9369'
+            data = self.create_fake_data(dt, dt + hour_delta, project_id)
+            self.storage.push(data, project_id)
+            dt += hour_delta
+
+
+class ScopeStateFixture(fixture.GabbiFixture):
+
+    def start_fixture(self):
+        self.sm = storage_state.StateManager()
+        self.sm.init()
+        data = [
+            ('aaaa', datetime.datetime(2019, 1, 1), 'fet1', 'col1', 'key1'),
+            ('bbbb', datetime.datetime(2019, 2, 2), 'fet1', 'col1', 'key2'),
+            ('cccc', datetime.datetime(2019, 3, 3), 'fet1', 'col2', 'key1'),
+            ('dddd', datetime.datetime(2019, 4, 4), 'fet1', 'col2', 'key2'),
+            ('eeee', datetime.datetime(2019, 5, 5), 'fet2', 'col1', 'key1'),
+            ('ffff', datetime.datetime(2019, 6, 6), 'fet2', 'col1', 'key2'),
+            ('gggg', datetime.datetime(2019, 6, 6), 'fet2', 'col2', 'key1'),
+            ('hhhh', datetime.datetime(2019, 6, 6), 'fet2', 'col2', 'key2'),
+        ]
+        for d in data:
+            self.sm.set_state(
+                d[0], d[1], fetcher=d[2], collector=d[3], scope_key=d[4])
+
+    def stop_fixture(self):
+        session = db.get_session()
+        q = utils.model_query(
+            self.sm.model,
+            session)
+        q.delete()
 
 
 class CORSConfigFixture(fixture.GabbiFixture):
@@ -397,11 +447,59 @@ class MetricsConfFixture(fixture.GabbiFixture):
         ck_utils.load_conf = self._original_function
 
 
-class SkipIfFunctional(fixture.GabbiFixture):
+class NowInfluxStorageDataFixture(NowStorageDataFixture):
 
     def start_fixture(self):
-        if is_functional_test():
-            raise SkipTest
+        cli = influx_utils.FakeInfluxClient()
+        st = storage.get_storage()
+        st._conn = cli
+
+        self._get_storage_patch = mock.patch(
+            'cloudkitty.storage.get_storage',
+            new=lambda **kw: st,
+        )
+        self._get_storage_patch.start()
+
+        super(NowInfluxStorageDataFixture, self).start_fixture()
+
+    def initialize_data(self):
+        data = test_utils.generate_v2_storage_data(
+            start=tzutils.get_month_start(),
+            end=tzutils.localized_now().replace(hour=0),
+        )
+        self.storage.push([data])
+
+    def stop_fixture(self):
+        self._get_storage_patch.stop()
+
+
+class InfluxStorageDataFixture(StorageDataFixture):
+
+    def start_fixture(self):
+        cli = influx_utils.FakeInfluxClient()
+        st = storage.get_storage()
+        st._conn = cli
+
+        self._get_storage_patch = mock.patch(
+            'cloudkitty.storage.get_storage',
+            new=lambda **kw: st,
+        )
+        self._get_storage_patch.start()
+
+        super(InfluxStorageDataFixture, self).start_fixture()
+
+    def stop_fixture(self):
+        self._get_storage_patch.stop()
+
+
+class UTCFixture(fixture.GabbiFixture):
+    """Set the local timezone to UTC"""
+    def start_fixture(self):
+        self._tzmock = mock.patch('cloudkitty.tzutils._LOCAL_TZ', tz.UTC)
+        self._tzmock.start()
+
+    def stop_fixture(self):
+        self._tzmock.stop()
 
 
 def setup_app():
