@@ -13,9 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
-import testtools
 import unittest
 
 import mock
@@ -23,7 +20,6 @@ from oslo_utils import uuidutils
 
 from cloudkitty.fetcher import keystone
 from cloudkitty import tests
-from cloudkitty.tests.utils import is_functional_test
 
 
 class FakeRole(object):
@@ -69,16 +65,15 @@ def Client(**kwargs):
     return FakeKeystoneClient(**kwargs)
 
 
-@testtools.skipIf(is_functional_test(), 'Not a functional test')
 class KeystoneFetcherTest(tests.TestCase):
     def setUp(self):
         super(KeystoneFetcherTest, self).setUp()
         self.conf.set_override('backend', 'keystone', 'tenant_fetcher')
-        self.conf.import_group('keystone_fetcher',
+        self.conf.import_group('fetcher_keystone',
                                'cloudkitty.fetcher.keystone')
 
     @unittest.SkipTest
-    def test_keystone_fetcher_filter_list(self):
+    def test_fetcher_keystone_filter_list(self):
         kclient = 'keystoneclient.client.Client'
         with mock.patch(kclient) as kclientmock:
             kclientmock.return_value = Client()
