@@ -13,12 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
 import copy
 import decimal
 import hashlib
-import testtools
 import zlib
 
 import mock
@@ -28,7 +25,6 @@ import six
 from cloudkitty.rating import pyscripts
 from cloudkitty.rating.pyscripts.db import api
 from cloudkitty import tests
-from cloudkitty.tests.utils import is_functional_test
 
 
 FAKE_UUID = '6c1b8a30-797f-4b7e-ad66-9879b79059fb'
@@ -86,11 +82,11 @@ CK_RESOURCES_DATA = [{
                     "unit": "instance"}}]}}]
 
 TEST_CODE1 = 'a = 1'.encode('utf-8')
-TEST_CODE1_CHECKSUM = hashlib.sha1(TEST_CODE1).hexdigest()
+TEST_CODE1_CHECKSUM = hashlib.sha512(TEST_CODE1).hexdigest()
 TEST_CODE2 = 'a = 0'.encode('utf-8')
-TEST_CODE2_CHECKSUM = hashlib.sha1(TEST_CODE2).hexdigest()
+TEST_CODE2_CHECKSUM = hashlib.sha512(TEST_CODE2).hexdigest()
 TEST_CODE3 = 'if a == 1: raise Exception()'.encode('utf-8')
-TEST_CODE3_CHECKSUM = hashlib.sha1(TEST_CODE3).hexdigest()
+TEST_CODE3_CHECKSUM = hashlib.sha512(TEST_CODE3).hexdigest()
 
 COMPLEX_POLICY1 = """
 import decimal
@@ -106,7 +102,6 @@ for period in data:
 """.encode('utf-8')
 
 
-@testtools.skipIf(is_functional_test(), 'Not a functional test')
 class PyScriptsRatingTest(tests.TestCase):
     def setUp(self):
         super(PyScriptsRatingTest, self).setUp()
@@ -241,7 +236,8 @@ class PyScriptsRatingTest(tests.TestCase):
             setattr,
             script,
             'checksum',
-            'da39a3ee5e6b4b0d3255bfef95601890afd80709')
+            'cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce4'
+            '7d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e')
 
     def test_update_checksum(self):
         self._db_api.create_script('policy1', TEST_CODE1)
@@ -284,7 +280,7 @@ class PyScriptsRatingTest(tests.TestCase):
             }}
         self.assertEqual(expected, self._pyscripts._scripts)
         context = {'a': 0}
-        exec(self._pyscripts._scripts[FAKE_UUID]['code'], context)
+        exec(self._pyscripts._scripts[FAKE_UUID]['code'], context)  # nosec
         self.assertEqual(1, context['a'])
 
     def test_update_script_on_checksum_change(self):
