@@ -14,17 +14,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
 import calendar
 import copy
 import csv
 import datetime
-import json
 import random
 import sys
 import uuid
 
+from cloudkitty import json_utils as json
+
 
 COMPUTE = {
     "type": "compute",
